import click
from .document import generate_document
from . import config


# Module API

@click.command(help='')
@click.argument('package')
@click.argument('document')
@click.option('--in-place', is_flag=True, show_default=True)
@click.option('--package-pattern', default=config.PACKAGE_PATTERN, show_default=True)
@click.option('--document-section', default=config.DOCUMENT_SECTION, show_default=True)
@click.version_option(config.VERSION, message='%(version)s')
def cli(package, document, **options):
    """ Command-line interface

    ```
    Usage: referencer [OPTIONS] PACKAGE DOCUMENT

    Options:
      --in-place               [default: False]
      --package-pattern TEXT   [default: from \\.(\\w+) import (\\w+)]
      --document-section TEXT  [default: ## Reference]
      --version                Show the version and exit.
      --help                   Show this message and exit.
    ```

    """
    in_place = options.pop('in_place')
    content = generate_document(package, document, **options)
    if in_place:
        with open(document, 'w') as file:
            file.write(content)
            return
    print(content)



from .reference import generate_reference
from . import config


# Module API

def generate_document(package, document,
        package_pattern=config.PACKAGE_PATTERN,
        document_section=config.DOCUMENT_SECTION):
    """ Generate document
    """
    content = ''
    replace = False
    reference = generate_reference(package, package_pattern)
    with open(document) as file:
        for line in file:
            if line.startswith('%s ' % document_section.split(' ')[0]):
                if replace:
                    content += "\n%s" % reference
                replace = False
            if line.startswith(document_section):
                content += line
                replace = True
            if replace:
                continue
            content += line
        if replace:
            content += "\n%s" % reference
    return content



import os
import io
from setuptools import setup, find_packages


# Helpers
def read(*paths):
    """Read a text file."""
    basedir = os.path.dirname(__file__)
    fullpath = os.path.join(basedir, *paths)
    contents = io.open(fullpath, encoding='utf-8').read().strip()
    return contents


# Prepare
PACKAGE = 'referencer'
INSTALL_REQUIRES = [
    'pydoc-markdown',
    'click',
]
TESTS_REQUIRE = [
    'mock',
    'pylama',
    'pytest',
    'pytest-cov',
]
README = read('README.md')
VERSION = read(PACKAGE, 'VERSION')
PACKAGES = find_packages(exclude=['examples', 'tests'])


# Run
setup(
    name=PACKAGE,
    version=VERSION,
    packages=PACKAGES,
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    tests_require=TESTS_REQUIRE,
    extras_require={
        'develop': TESTS_REQUIRE,
    },
    entry_points={
        'console_scripts': [
            'referencer = referencer.__main__:cli',
        ]
    },
    zip_safe=False,
    long_description=README,
    long_description_content_type='text/markdown',
    description='Generate a markfown reference from your public API docstrings and add it to your readme',
    author='Evgeny Karev',
    author_email='eskarev@gmail.com',
    url='https://github.com/roll/referencer-py',
    license='MIT',
    keywords=[
        'docstring',
        'markdown',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



import re
import subprocess
from . import config


# Module API

def generate_reference(package,
        package_pattern=config.PACKAGE_PATTERN):
    """ Generate reference
    """
    content = ''
    with open('%s/__init__.py' % package) as file:
        for line in file:
            match = re.match(package_pattern, line)
            if not match:
                continue
            capture = False
            module, object = match.groups()
            is_class = object[0].isupper()
            docs = subprocess.getoutput('pydocmd simple %s.%s++' % (package, module))
            docs = re.sub(r'\n\n-', r'\n-', docs)
            docs = re.sub(r'\\:', r':', docs)
            for line in docs.split("\n"):
                if line.startswith('## '):
                    if capture:
                        break
                if line == '## %s' % object:
                    capture = True
                if not capture:
                    continue
                if is_class:
                    line = re.sub(r'^### (.*)', r'### %s.\1' % object.lower(), line)
                    line = re.sub(r'^%s\.' % object, r'%s.' % object.lower(), line)
                line = re.sub(r'^(###?) ([\w.]+)', r'#\1 `\2`', line)
                content += "%s\n" % line
    return content
